@@ -64,17 +64,19 @@ class FirewallCommand(object):
     def print_warning(self, msg=None):
         FAIL = '\033[91m'
         END = '\033[00m'
-        self.print_error_msg(FAIL + msg + END)
+        if sys.stderr.isatty():
+            msg = FAIL + msg + END
+        self.print_error_msg(msg)
 
     def print_and_exit(self, msg=None, exit_code=0):
         #OK = '\033[92m'
-        FAIL = '\033[91m'
-        END = '\033[00m'
+        #END = '\033[00m'
         if exit_code > 1:
-            self.print_error_msg(FAIL + msg + END)
+            self.print_warning(msg)
         else:
+            #if sys.stdout.isatty():
+            #   msg = OK + msg + END
             self.print_msg(msg)
-            #self.print_msg(OK + msg + END)
         sys.exit(exit_code)
 
     def fail(self, msg=None):
@@ -84,8 +86,8 @@ class FirewallCommand(object):
         if msg is not None and self.verbose:
             sys.stdout.write(msg + "\n")
 
-    def __cmd_sequence(self, cmd_type, option, action_method, query_method,
-                       parse_method, message, start_args=None, end_args=None,
+    def __cmd_sequence(self, cmd_type, option, action_method, query_method, # pylint: disable=W0613, R0913, R0914
+                       parse_method, message, start_args=None, end_args=None, # pylint: disable=W0613
                        no_exit=False):
         if self.fw is not None:
             self.fw.authorizeAll()
@@ -130,7 +132,7 @@ class FirewallCommand(object):
                     msg = str(msg)
                 code = FirewallError.get_code(msg)
                 if code in [ errors.ALREADY_ENABLED, errors.NOT_ENABLED,
-                             errors.ZONE_ALREADY_SET ]:
+                             errors.ZONE_ALREADY_SET, errors.ALREADY_SET ]:
                     code = 0
                 if len(option) > 1:
                     self.print_warning("Warning: %s" % msg)
@@ -159,37 +161,37 @@ class FirewallCommand(object):
                 # INVALID_PORT and INVALID_PROTOCOL.
                 sys.exit(errors.UNKNOWN_ERROR)
 
-    def add_sequence(self, option, action_method, query_method, parse_method,
+    def add_sequence(self, option, action_method, query_method, parse_method, # pylint: disable=R0913
                      message, no_exit=False):
         self.__cmd_sequence("add", option, action_method, query_method,
                             parse_method, message, no_exit=no_exit)
 
-    def x_add_sequence(self, x, option, action_method, query_method,
+    def x_add_sequence(self, x, option, action_method, query_method, # pylint: disable=R0913
                        parse_method, message, no_exit=False):
         self.__cmd_sequence("add", option, action_method, query_method,
                             parse_method, message, start_args=[x],
                             no_exit=no_exit)
 
-    def zone_add_timeout_sequence(self, zone, option, action_method,
+    def zone_add_timeout_sequence(self, zone, option, action_method, # pylint: disable=R0913
                                   query_method, parse_method, message,
                                   timeout, no_exit=False):
         self.__cmd_sequence("add", option, action_method, query_method,
                             parse_method, message, start_args=[zone],
                             end_args=[timeout], no_exit=no_exit)
 
-    def remove_sequence(self, option, action_method, query_method,
+    def remove_sequence(self, option, action_method, query_method, # pylint: disable=R0913
                         parse_method, message, no_exit=False):
         self.__cmd_sequence("remove", option, action_method, query_method,
                             parse_method, message, no_exit=no_exit)
 
-    def x_remove_sequence(self, x, option, action_method, query_method,
+    def x_remove_sequence(self, x, option, action_method, query_method, # pylint: disable=R0913
                           parse_method, message, no_exit=False):
         self.__cmd_sequence("remove", option, action_method, query_method,
                             parse_method, message, start_args=[x],
                             no_exit=no_exit)
 
 
-    def __query_sequence(self, option, query_method, parse_method, message,
+    def __query_sequence(self, option, query_method, parse_method, message, # pylint: disable=R0913
                          start_args=None, no_exit=False):
         items = [ ]
         for item in option:
@@ -225,6 +227,12 @@ class FirewallCommand(object):
                 else:
                     self.print_and_exit("Error: %s" % msg.get_dbus_message(),
                                         code)
+            except Exception as msg:
+                code = FirewallError.get_code(str(msg))
+                if len(option) > 1:
+                    self.print_warning("Warning: %s" % msg)
+                else:
+                    self.print_and_exit("Error: %s" % msg, code)
             self.activate_exception_handler()
             if len(option) > 1:
                 self.print_msg("%s: %s" % (message % item, ("no", "yes")[res]))
@@ -233,12 +241,12 @@ class FirewallCommand(object):
         if not no_exit:
             sys.exit(0)
 
-    def query_sequence(self, option, query_method, parse_method, message,
+    def query_sequence(self, option, query_method, parse_method, message, # pylint: disable=R0913
                        no_exit=False):
         self.__query_sequence(option, query_method, parse_method,
                               message, no_exit=no_exit)
 
-    def x_query_sequence(self, x, option, query_method, parse_method,
+    def x_query_sequence(self, x, option, query_method, parse_method, # pylint: disable=R0913
                          message, no_exit=False):
         self.__query_sequence(option, query_method, parse_method,
                               message, start_args=[x], no_exit=no_exit)
@@ -261,31 +269,41 @@ class FirewallCommand(object):
                                 "portid[-portid]%sprotocol" % separator)
         if not check_port(port):
             raise FirewallError(errors.INVALID_PORT, port)
-        if proto not in [ "tcp", "udp" ]:
+        if proto not in [ "tcp", "udp", "sctp", "dccp" ]:
             raise FirewallError(errors.INVALID_PROTOCOL,
-                                "'%s' not in {'tcp'|'udp'}" % proto)
+                                "'%s' not in {'tcp'|'udp'|'sctp'|'dccp'}" % \
+                                proto)
         return (port, proto)
 
-    def parse_forward_port(self, value):
+    def parse_forward_port(self, value, compat=False):
         port = None
         protocol = None
         toport = None
         toaddr = None
-        args = value.split(":")
-        for arg in args:
-            try:
-                (opt, val) = arg.split("=")
-                if opt == "port":
-                    port = val
-                elif opt == "proto":
-                    protocol = val
-                elif opt == "toport":
-                    toport = val
-                elif opt == "toaddr":
-                    toaddr = val
-            except ValueError:
+        i = 0
+        while ("=" in value[i:]):
+            opt = value[i:].split("=", 1)[0]
+            i += len(opt) + 1
+            if "=" in value[i:]:
+                val = value[i:].split(":", 1)[0]
+            else:
+                val = value[i:]
+            i += len(val) + 1
+
+            if opt == "port":
+                port = val
+            elif opt == "proto":
+                protocol = val
+            elif opt == "toport":
+                toport = val
+            elif opt == "toaddr":
+                toaddr = val
+            elif opt == "if" and compat:
+                # ignore if option in compat mode
+                pass
+            else:
                 raise FirewallError(errors.INVALID_FORWARD,
-                                    "invalid forward port arg '%s'" % (arg))
+                                    "invalid forward port arg '%s'" % (opt))
         if not port:
             raise FirewallError(errors.INVALID_FORWARD, "missing port")
         if not protocol:
@@ -295,13 +313,15 @@ class FirewallCommand(object):
 
         if not check_port(port):
             raise FirewallError(errors.INVALID_PORT, port)
-        if protocol not in [ "tcp", "udp" ]:
+        if protocol not in [ "tcp", "udp", "sctp", "dccp" ]:
             raise FirewallError(errors.INVALID_PROTOCOL,
-                                "'%s' not in {'tcp'|'udp'}" % protocol)
+                                "'%s' not in {'tcp'|'udp'|'sctp'|'dccp'}" % \
+                                protocol)
         if toport and not check_port(toport):
             raise FirewallError(errors.INVALID_PORT, toport)
         if toaddr and not check_single_address("ipv4", toaddr):
-            raise FirewallError(errors.INVALID_ADDR, toaddr)
+            if compat or not check_single_address("ipv6", toaddr):
+                raise FirewallError(errors.INVALID_ADDR, toaddr)
 
         return (port, protocol, toport, toaddr)
 
@@ -357,11 +377,8 @@ class FirewallCommand(object):
                                 "Module name '%s' too short" % value)
         return value
 
-    def print_zone_info(self, zone, settings, default_zone=None):
+    def print_zone_policy_info(self, zone, settings, default_zone=None, extra_interfaces=[], isPolicy=True): # pylint: disable=R0914
         target = settings.getTarget()
-        icmp_block_inversion = settings.getIcmpBlockInversion()
-        interfaces = settings.getInterfaces()
-        sources = settings.getSources()
         services = settings.getServices()
         ports = settings.getPorts()
         protocols = settings.getProtocols()
@@ -372,12 +389,37 @@ class FirewallCommand(object):
         rules = settings.getRichRules()
         description = settings.getDescription()
         short_description = settings.getShort()
+        if isPolicy:
+            ingress_zones = settings.getIngressZones()
+            egress_zones = settings.getEgressZones()
+            priority = settings.getPriority()
+        else:
+            icmp_block_inversion = settings.getIcmpBlockInversion()
+            interfaces = sorted(set(settings.getInterfaces() + extra_interfaces))
+            sources = settings.getSources()
+            forward = settings.getForward()
+            ingress_priority = settings.getIngressPriority()
+            egress_priority = settings.getEgressPriority()
+
+        def rich_rule_sorted_key(rule):
+            priority = 0
+            search_str = "priority="
+            try:
+                i = rule.index(search_str)
+            except ValueError:
+                pass
+            else:
+                i += len(search_str)
+                priority = int(rule[i:i+(rule[i:].index(" "))].replace("\"", ""))
+
+            return priority
 
         attributes = []
         if default_zone is not None:
             if zone == default_zone:
                 attributes.append("default")
-        if interfaces or sources:
+        if (not isPolicy and (interfaces or sources)) or \
+           (    isPolicy and ingress_zones and egress_zones):
             attributes.append("active")
         if attributes:
             zone = zone + " (%s)" % ", ".join(attributes)
@@ -385,17 +427,28 @@ class FirewallCommand(object):
         if self.verbose:
             self.print_msg("  summary: " + short_description)
             self.print_msg("  description: " + description)
+        if isPolicy:
+            self.print_msg("  priority: " + str(priority))
         self.print_msg("  target: " + target)
-        self.print_msg("  icmp-block-inversion: %s" % \
-                       ("yes" if icmp_block_inversion else "no"))
-        self.print_msg("  interfaces: " + " ".join(interfaces))
-        self.print_msg("  sources: " + " ".join(sources))
-        self.print_msg("  services: " + " ".join(services))
+        if not isPolicy:
+            self.print_msg("  ingress-priority: " + str(ingress_priority))
+            self.print_msg("  egress-priority: " + str(egress_priority))
+            self.print_msg("  icmp-block-inversion: %s" % \
+                           ("yes" if icmp_block_inversion else "no"))
+        if isPolicy:
+            self.print_msg("  ingress-zones: " + " ".join(ingress_zones))
+            self.print_msg("  egress-zones: " + " ".join(egress_zones))
+        else:
+            self.print_msg("  interfaces: " + " ".join(interfaces))
+            self.print_msg("  sources: " + " ".join(sources))
+        self.print_msg("  services: " + " ".join(sorted(services)))
         self.print_msg("  ports: " + " ".join(["%s/%s" % (port[0], port[1])
                                                for port in ports]))
-        self.print_msg("  protocols: " + " ".join(protocols))
+        self.print_msg("  protocols: " + " ".join(sorted(protocols)))
+        if not isPolicy:
+            self.print_msg("  forward: %s" % ("yes" if forward else "no"))
         self.print_msg("  masquerade: %s" % ("yes" if masquerade else "no"))
-        self.print_msg("  forward-ports: " +
+        self.print_msg("  forward-ports: " + ("\n\t" if forward_ports else "") +
                        "\n\t".join(["port=%s:proto=%s:toport=%s:toaddr=%s" % \
                                     (port, proto, toport, toaddr)
                                     for (port, proto, toport, toaddr) in \
@@ -404,7 +457,14 @@ class FirewallCommand(object):
                        " ".join(["%s/%s" % (port[0], port[1])
                                  for port in source_ports]))
         self.print_msg("  icmp-blocks: " + " ".join(icmp_blocks))
-        self.print_msg("  rich rules: \n\t" + "\n\t".join(rules))
+        self.print_msg("  rich rules: " + ("\n\t" if rules else "") +
+                            "\n\t".join(sorted(rules, key=rich_rule_sorted_key)))
+
+    def print_zone_info(self, zone, settings, default_zone=None, extra_interfaces=[]):
+        self.print_zone_policy_info(zone, settings, default_zone=default_zone, extra_interfaces=extra_interfaces, isPolicy=False)
+
+    def print_policy_info(self, policy, settings, default_zone=None, extra_interfaces=[]):
+        self.print_zone_policy_info(policy, settings, default_zone=default_zone, extra_interfaces=extra_interfaces, isPolicy=True)
 
     def print_service_info(self, service, settings):
         ports = settings.getPorts()
@@ -414,6 +474,8 @@ class FirewallCommand(object):
         description = settings.getDescription()
         destinations = settings.getDestinations()
         short_description = settings.getShort()
+        includes = settings.getIncludes()
+        helpers = settings.getHelpers()
         self.print_msg(service)
         if self.verbose:
             self.print_msg("  summary: " + short_description)
@@ -428,6 +490,8 @@ class FirewallCommand(object):
         self.print_msg("  destination: " +
                        " ".join(["%s:%s" % (k, v)
                                  for k, v in destinations.items()]))
+        self.print_msg("  includes: " + " ".join(sorted(includes)))
+        self.print_msg("  helpers: " + " ".join(sorted(helpers)))
 
     def print_icmptype_info(self, icmptype, settings):
         destinations = settings.getDestinations()
@@ -483,7 +547,7 @@ class FirewallCommand(object):
         self.fail_if_not_authorized(exception_message)
         code = FirewallError.get_code(str(exception_message))
         if code in [ errors.ALREADY_ENABLED, errors.NOT_ENABLED,
-                     errors.ZONE_ALREADY_SET ]:
+                     errors.ZONE_ALREADY_SET, errors.ALREADY_SET ]:
             self.print_warning("Warning: %s" % exception_message)
         else:
             self.print_and_exit("Error: %s" % exception_message, code)
@@ -502,6 +566,7 @@ class FirewallCommand(object):
 
     def get_ipset_entries_from_file(self, filename):
         entries = [ ]
+        entries_set = set()
         f = open(filename)
         for line in f:
             if not line:
@@ -509,7 +574,8 @@ class FirewallCommand(object):
             line = line.strip()
             if len(line) < 1 or line[0] in ['#', ';']:
                 continue
-            if line not in entries:
+            if line not in entries_set:
                 entries.append(line)
+                entries_set.add(line)
         f.close()
         return entries
