@@ -40,7 +40,7 @@ APPLET_NAME = 'firewall-applet'
 DATADIR = '/usr/share/' + DAEMON_NAME
 CONFIG_GLADE_NAME = CONFIG_NAME + '.glade'
 COPYRIGHT = '(C) 2010-2017 Red Hat, Inc.'
-VERSION = '1.3.4'
+VERSION = '2.0.0'
 AUTHORS = [
     "Thomas Woerner <twoerner@redhat.com>",
     "Jiri Popelka <jpopelka@redhat.com>",
@@ -113,8 +113,8 @@ COMMANDS = {
     "eb":           "/usr/sbin/ebtables",
     "eb-restore":   "/usr/sbin/ebtables-restore",
     "ipset":        "/usr/sbin/ipset",
-    "modprobe":     "/sbin/modprobe",
-    "rmmod":        "/sbin/rmmod",
+    "modprobe":     "/usr/sbin/modprobe",
+    "rmmod":        "/usr/sbin/rmmod",
 }
 
 LOG_DENIED_VALUES = [ "all", "unicast", "broadcast", "multicast", "off" ]
@@ -135,3 +135,5 @@ FALLBACK_FIREWALL_BACKEND = "nftables"
 FALLBACK_FLUSH_ALL_ON_RELOAD = True
 FALLBACK_RFC3964_IPV4 = True
 FALLBACK_ALLOW_ZONE_DRIFTING = False
+FALLBACK_NFTABLES_FLOWTABLE = "off"
+FALLBACK_NFTABLES_COUNTERS = False
