@@ -31,7 +31,7 @@ valid_keys = [ "DefaultZone", "MinimalMark", "CleanupOnExit",
                "CleanupModulesOnExit", "Lockdown", "IPv6_rpfilter",
                "IndividualCalls", "LogDenied", "AutomaticHelpers",
                "FirewallBackend", "FlushAllOnReload", "RFC3964_IPv4",
-               "AllowZoneDrifting" ]
+               "AllowZoneDrifting", "NftablesFlowtable", "NftablesCounters"]
 
 class firewalld_conf(object):
     def __init__(self, filename):
@@ -79,6 +79,8 @@ class firewalld_conf(object):
         self.set("FlushAllOnReload", "yes" if config.FALLBACK_FLUSH_ALL_ON_RELOAD else "no")
         self.set("RFC3964_IPv4", "yes" if config.FALLBACK_RFC3964_IPV4 else "no")
         self.set("AllowZoneDrifting", "yes" if config.FALLBACK_ALLOW_ZONE_DRIFTING else "no")
+        self.set("NftablesFlowtable", config.FALLBACK_NFTABLES_FLOWTABLE)
+        self.set("NftablesCounters", "yes" if config.FALLBACK_NFTABLES_COUNTERS else "no")
 
     # load self.filename
     def read(self):
